define([
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojox/mvc/getPlainValue",
        "dojox/mvc/at",
        "dojo/promise/all",
        "dojox/mvc/getStateful",
        "dojox/mvc/StatefulArray",
        "aps/ResourceStore",
        "aps/Memory",
        "aps/TextBox",
        "aps/json!./wizard/newvps.json",
        "aps/View"
    ],
    function (declare, lang, getPlainValue, at, all, getStateful, StatefulArray,
              Store, Memory, TextBox, newVPS, View) {

        /* Declare global variables */
        var self,
            offerById = {};
        /* Define a global function to request the platform for creation of new VPSes */
        function resourceRequest(tileId, offerById, view, subscriptionId) {
            // Send a Biz API request for resource usage change
            var offerId = view.byId(tileId).get('offerId');
            var offer = offerById[offerId];
            var vpsModel = getStateful({"data": newVPS}); // Default VPS properties
            vpsModel.data.set({
                offer: {aps: {id: offerId}},
                platform: offer.platform,
                hardware: offer.hardware
            });
            var usersQuery = view.byId("addUsers_grid").get("store").query();
            all({
                users: usersQuery,
                total: usersQuery.total
            }).then(function(data) {
                var users = data.users,
                    usersTotal = data.total;

                var deltas = [
                    {
                        apsType: "http://aps-standard.org/samples/suwizard1p/vps/1.0",
                        delta: usersTotal
                    },
                    {
                        apsType: "http://aps-standard.org/samples/suwizard1p/offer/1.0",
                        apsId: offerId,
                        delta: usersTotal
                    }
                ];

                /* Call the APS Biz API to provision VPSes for the new users*/
                var request = {};
                request[subscriptionId] = {
                    deltas: deltas,
                    operations: users.map(function(user) {
                        return {
                            provision: lang.mixin(getPlainValue(vpsModel.data), {
                                name: user.hostName,
                                user: {aps: {id: user.id}},
                                userName: user.name
                            })
                        };
                    })
                };
                aps.biz.requestUsageChange(request).then(null, function (err) {
                    console.error(err);     // For debugging only
                });
            });
        }

        return declare(View, {
            init: function() {
                self = this;

                /* Render a gird cell to allow entering a host name */
                var renderHostName = function(row) {
                    return new TextBox({
                        value: at(row, "hostName"),
                        required: true
                    });
                };

                /* Set a watcher for selection of a tile */
                var selectedTiles = new StatefulArray();
                selectedTiles.watchElements(function(index, removals, adds){
                    if(adds.length == 1) resourceRequest(
                        adds[0],
                        offerById,
                        self,
                        aps.context.subscriptionId
                    );
                });

                /* Define and return widgets */
                return [
                    ["aps/Grid",{
                        id: this.genId("addUsers_grid"),
                        store: new Memory({data: [], idProperty: "id"}),
                        columns: [
                            { name: "User Name", field: "name" },
                            { name: "Host Name", field: "hostName", renderCell: renderHostName }
                        ]
                    }],
                    ["aps/Tiles", {
                        id: this.genId("addUsers_offerSet"),
                        title: "Select an offer",
                        selectionMode: "single",
                        required: true,
                        store: new Memory({data: []}),
                        selectionArray: selectedTiles
                    }, [
                        [ "aps/Tile", {
                            offerId: at("rel:", "id"),
                            id: this.genId("${offerId}"),
                            gridSize: "md-4 xs-12",
                            title: at("rel:", "name")
                        }, [
                            [ "aps/Output", {
                                value: at("rel:", "os"),
                                content: "Operating System: ${value}"
                            }],
                            [ "aps/Output", {
                                value: at("rel:", "cpu"),
                                content: "CPU cores: ${value}"
                            }],
                            [ "aps/Output", {
                                value: at("rel:", "memory"),
                                content: "Memory: ${value} MB"
                            }],
                            [ "aps/Output", {
                                value: at("rel:", "diskspace"),
                                content: "Disk space: ${value} GB"
                            }],
                            [ "aps/Output", {
                                value: at("rel:", "priceText")
                            }],
                            [ "aps/Output", {
                                usage: at("rel:", "usage"),
                                limit: at("limit"),
                                content: "${usage} of ${limit} servers are used."
                            }]
                        ]]
                    ]]
                ];

            }, // End of Init

            /* Process data available at the onContext phase */
            onContext: function() {
                aps.context.subscriptionId = aps.context.vars.context.aps.subscription;
                /* Define data for the APS Biz API */
                var offerStore = new Store({
                    target: "/aps/2/resources",
                    apsType: "http://aps-standard.org/samples/suwizard1p/offer/1.0"
                });
                var offerFilter = { filter:
                        [{ apsType: "http://aps-standard.org/samples/suwizard1p/offer/1.0" }]
                };
                var parentContainer = self.byId("addUsers_offerSet"); // ``aps/Tiles``

                /* Split into 2 cases */
                /* First case - I'm back from another wizard step */
                if (aps.context.wizardData[
                    "http://aps-standard.org/samples/suwizard1p#add-user-service"
                    ]) {
                    all({
                        newUsers: aps.biz.getResourcesToBind(),
                        oldUsers: self.byId("addUsers_grid").get("store").query()
                    }).then(function(data) {
                        var newUsers = data.newUsers.users,
                            oldUsers = data.oldUsers;

                        /* User processing */
                        var users = newUsers.map(function(user) {
                            var oldUser = oldUsers.find(function(oldUser) {
                                return oldUser.id === user.aps.id;
                            });

                            return getStateful({
                                id: user.aps.id,
                                name: user.fullName,
                                hostName: oldUser ? oldUser.hostName : ""
                            });
                        });
                        var usersStore = new Memory({data: users, idProperty: "id"});
                        self.byId("addUsers_grid").set("store", usersStore);

                        /* Check if the default Offer is available for the updated user list */
                        resourceRequest(parentContainer.get(
                            "selectionArray")[0],
                            offerById,
                            self,
                            aps.context.subscriptionId
                        );

                    }).then(function() {
                        aps.apsc.hideLoading();
                    });
                }

                /* Second case - I'm here for the first time. */
                else {
                    /* Retrieve the resources to be processed */
                    all({
                        newUsers: aps.biz.getResourcesToBind(), // New users
                        subscribedOffers: aps.biz.getResourcesInfo(offerFilter), // Subscribed offers
                        allOffers: offerStore.query()  // All offers of the application
                    }).then(function(data) {
                        var newUsers = data.newUsers,
                            subscribedOffers = data.subscribedOffers,
                            allOffers = data.allOffers;

                        if(subscribedOffers.length === 0) return; // No offers in the subscription

                        /* User processing */
                        var users = newUsers.users.map(function(user) {
                            return getStateful({
                                id: user.aps.id,
                                name: user.fullName,
                                hostName: ""
                            });
                        });
                        var usersStore = new Memory({data: users, idProperty: "id"});
                        self.byId("addUsers_grid").set("store", usersStore);

                        /* Fill in the Offer-by-ID dictionary */
                        allOffers.forEach(function(offer) {
                            offerById[offer.aps.id] = offer;
                        });

                        /* Define the offer selection list */
                        var offers = subscribedOffers.map(function(offer) {
                            var offerApsResource = offerById[offer.apsId];
                            return {
                                id: offer.apsId,
                                name: offerApsResource.name,
                                os: offerApsResource.platform.OS.name,
                                cpu: offerApsResource.hardware.CPU.number,
                                memory: offerApsResource.hardware.memory,
                                diskspace: offerApsResource.hardware.diskspace,
                                limit: offer.limit,
                                usage: offer.usage,
                                priceText: offer.priceText
                            };
                        });
                        var offerCache = new Memory({data: offers, idProperty: "id"});

                        /* Define the tiles presenting the subscribed offers */
                        parentContainer.removeAll();        // Remove all child tiles
                        parentContainer.set('store', offerCache);

                        /* Check if the default Offer is available for all new users */
                        resourceRequest(
                            parentContainer.get("selectionArray")[0],
                            offerById,
                            self,
                            aps.context.subscriptionId
                        );

                    })// End of all-then callback function

                    /* Hide loading once all async operations are completed */
                        .then(function() {
                            aps.apsc.hideLoading();
                        });
                }

            }, // End of onContext

            onNext: function() {
                // Go to the next wizard step
                var page = this.byId("apsPageContainer");
                page.get("messageList").removeAll();
                if (!page.validate()) {
                    aps.apsc.cancelProcessing();
                    return;
                }
                aps.apsc.next("I was here"); // Flag for the case I'll be back
            },

            onPrev: function() {
                // Return to the previous step
                aps.apsc.prev("I was here"); // Flag for the case I'll be back
            }

        });  // End of Declare
});     // End of Define