define([
    "aps/nav/ViewPlugin",
    "dojo/_base/declare",
    "dojo/Deferred",
    "dojox/mvc/getStateful",
    "aps/FieldSet",
    "aps/Container",
    "aps/Output",
    "aps/Gauge"
], function(ViewPlugin, declare, Deferred, getStateful, FieldSet, Container, Output, Gauge) {

    return declare(ViewPlugin, {
        init: function(mediator) {

            /* Define the function that returns the title of the service for the Grid */
            mediator.getServicesList = function() {
                return { "title": "VPSes" };
            };

            /* The main function that defines UI elements presenting
               the services assigned to users */

            mediator.getState = function(users) {

                /* Return a list of user APS IDs mapped to an array of <state> objects */
                var userStates =  users.reduce(function(userServiceMap, user) {

                    /* Process each user and add the result to userServiceMap */
                    var state = {};
                    state.status = "Not Assigned";    // Initially, suppose so
                    state.title = "VPSes";            // Title in a tile
                    var vpses = 0,                    // Number of VPSes assigned to the user
                        stopped = 0,                   // Number of stopped VPSes
                        running = 0;                   // Number of running VPSes

                    /* Initialize the state object and resource counters */
                    user.services.forEach(function(service) {
                        if (service.aps.type === "http://aps-standard.org/samples/suwizard1p/vps/1.0") {
                            state.status = "Assigned";
                            vpses += 1;
                            if (service.state === "Stopped") {
                                stopped += 1;
                            }
                            else running += 1;
                        }
                    });

                    /* Figure out service resources assigned to each user
                        and define the state.widget */
                    if(state.status === "Assigned") {
                        if (!state.widget) {
                            var vpsOutput = new Container();
                            vpsOutput.addChild(new Output({
                                value: vpses,
                                running: running,
                                stopped: stopped,
                                content: "Assigned: ${value} <br/> Running: ${running} <br/> Stopped: ${stopped}"
                            }));
                            state.widget = vpsOutput;
                        }

                        /* Continue with definition of the state.gridWidget object */
                        if(!state.gridWidget) {
                            var vpsGridOutput = new Gauge({
                                legend : "${value} of ${maximum} are stopped",
                                minimum : 0,
                                maximum : vpses,
                                value: stopped
                            });
                            state.gridWidget = vpsGridOutput;
                        }  // End of if(!state.gridWidget)

                    }

                    /* Return a list of user APS IDs mapped to an array of <state> objects */
                    userServiceMap[user.aps.id] = [state];
                    return userServiceMap;
                }, {}); // {} is the initial value of the userServiceMap set

                return userStates;
            };    // End of getState method
        }        // End of Init
    });         // End of Declare
});
