define([
   "dojo/_base/declare",
   "dojo/when",
   "dojox/mvc/at",
   "dojox/mvc/getStateful",
   "dojo/promise/all",
   "dojox/mvc/getPlainValue",
   "aps/View",
   "aps/Memory",
   "aps/ResourceStore",
   
   /* Include JSON files that defines the default properties */
   "aps/json!./newvps.json",
   "aps/json!app/newoffer.json"
],
function (declare, when, at, getStateful, all, getPlainValue, View, Memory, Store, newVPS, newOffer) {
    var self,
        wizardData;
    return declare(View, {
        init: function() {
            self = this;
            /* Declare data sources */
            self.userSelectionList = new Memory({data: [], idProperty: "aps.id"});
            self.offerCache = new Memory({
               data: [],
               idProperty: "aps.id"
            });
            self.offerCtrl = getStateful({
               data: newOffer
            });
            self.vpsModel = getStateful({"data": newVPS});

            function onChange(offerID) {

                when(self.offerCache.get(offerID), function (data) {
                    self.offerCtrl.set("data", data);  // Update the current offer
                    self.vpsModel.data.platform.OS.set("name", self.offerCtrl.data.platform.OS.name);
                });
                self.vpsModel.data.set("userName", self.userSelectionList.get(self.vpsModel.data.user.aps.id).fullName);

                aps.biz.requestUsageChange (
                    { deltas: [{
                            apsType: "http://aps-standard.org/samples/suwizard1p/offer/1.0",
                            apsId: offerID,
                            delta: 1
                        }, {
                            apsType: "http://aps-standard.org/samples/suwizard1p/vps/1.0",
                            delta: 1
                    }],
                    operations: [{
                        provision: getPlainValue(self.vpsModel.data)
                        }
                    ]
                });
            }

            /* Define and return widgets */
            return ["aps/Panel", {id: this.genId("srvNew_form")}, [
               ["aps/FieldSet", {title: "General"}, [
                    ["aps/TextBox", {
                        label: "Server Name",
                        value: at(self.vpsModel.data, "name"),
                        required: true
                    }],
                    ["aps/TextBox", {
                        label: "Description",
                        value: at(self.vpsModel.data, "description")
                    }]
               ]],
               ["aps/Panel", {id: this.genId("srvNew1_offers_panel"), title: "Offering"}, [
                    ["aps/FieldSet", {id: this.genId("srvNew1_offers")}, [
                        ["aps/Select", {
                            id: self.genId("srvNew1_offerSelect"),
                            gridSize: 'md-3 xs-12',
                            store: self.offerCache,
                            required: true,
                            label: "Service profile:",
                            labelAttr: "name",
                            value: at(self.vpsModel.data.offer.aps, "id"),
                            onChange: onChange
                        }],
                        ["aps/Select", {
                            id: this.genId("srvNew1_userSelect"),
                            gridSize: 'md-3 xs-12',
                            label: _("VPS Owner"),
                            store: self.userSelectionList,
                            labelAttr: "fullName",
                            value: at(self.vpsModel.data.user.aps, "id"),
                            required: true
                        }]
                    ]]
                ]],
                ["aps/Panel", {id: this.genId("srvNew1_vps_configuration"), title: "Server Configuration"}, [
                   ["aps/FieldSet", {id: "srvNew1_config"}, [
                        ["aps/Output", {
                        id: this.genId("new1_os"),
                        label: "Operating System:",
                        value: at(self.offerCtrl.data.platform.OS, "name")
                        }],
                        ["aps/Spinner", {
                            label:  "CPU Number",
                            gridSize: 'md-3 xs-12',
                            minimum: 1,
                            maximum: at(self.offerCtrl.data.hardware.CPU, "number"),
                            value: at(self.vpsModel.data.hardware.CPU, "number"),
                            step: 1
                        }],
                        ["aps/Spinner", {
                            label: "Disk Space",
                            gridSize: 'md-3 xs-12',
                            minimum: 1,
                            maximum: at(self.offerCtrl.data.hardware, "diskspace"),
                            value: at(self.vpsModel.data.hardware, "diskspace"),
                            legend: "Gb",
                            step: 1
                        }],
                        ["aps/Spinner", {
                            label: "RAM",
                            gridSize: 'md-3 xs-12',
                            minimum: 128,
                            maximum: at(self.offerCtrl.data.hardware, "memory"),
                            value: at(self.vpsModel.data.hardware, "memory"),
                            legend: "Mb",
                            step: 128
                        }]
                   ]]
               ]]
            ]];
        },

        /* Additional single-page view flow method */
        onContext: function() {
            /* I'm here for the first time */
            if (typeof aps.context.wizardData === 'undefined' ||
                    !aps.context.wizardData["http://aps-standard.org/samples/suwizard1p#server-new-1"]) {
                var offerStore = new Store({ // The list of offers for selection
                    apsType: "http://aps-standard.org/samples/suwizard1p/offer/1.0",
                    target: "/aps/2/resources/"
                });
                var userStore = new Store({    // The list of users for selection
                    apsType: "http://aps-standard.org/types/core/user/1.0",
                    target: "/aps/2/resources/"
                });

                /* Get data from the wizard */
                wizardData = aps.context.wizardData["http://aps-standard.org/samples/suwizard1p#vps-wizard"];
                aps.context.subscriptionId = wizardData["subscriptionId"];
                if(wizardData["userId"]) {
                    self.vpsModel.data.user.aps.set(
                        "id",
                        wizardData["userId"]
                    );
                }

                all({
                    offers: offerStore.query(),
                    users: userStore.query()
                }).then(function(data) {
                    var offers = data.offers,
                    users = data.users;

                    if (offers.length === 0) return;
                    self.userSelectionList.setData(users);

                    self.offerCache.setData(offers);
                    if (aps.app.isNew !== false) {
                        self.offerCtrl.data.set(offers[0]);
                    }

                    self.vpsModel.data.offer.aps.set("id", self.offerCtrl.data.aps.id);
                    self.byId("srvNew1_userSelect").set("store", self.userSelectionList);
                    self.byId("srvNew1_offerSelect").set("store", self.offerCache);
                }).then(function() {
                    aps.apsc.hideLoading();
                });
            }
             /* I'm back from a neighbor step */
            else {
                aps.context.subscriptionId = wizardData["subscriptionId"];
                aps.apsc.hideLoading();
            }
        },

        /* Create handlers for the navigation buttons */
        onNext: function() {
            var page = this.byId("apsPageContainer");
            page.get("messageList").removeAll();
            if (!page.validate()) {
                aps.apsc.cancelProcessing();
                return;
            }

            /* Update all properties in the request */
            self.vpsModel.data.set("userName", self.userSelectionList.get(self.vpsModel.data.user.aps.id).fullName);
            aps.biz.requestUsageChange ({
                deltas: [{
                    apsType: "http://aps-standard.org/samples/suwizard1p/offer/1.0",
                    apsId: this.vpsModel.data.offer.aps.id,
                    delta: 1
                }, {
                    apsType: "http://aps-standard.org/samples/suwizard1p/vps/1.0",
                    delta: 1
                }],
                operations: [{
                    provision: getPlainValue(self.vpsModel.data)
                }]
            }).then(function() {
                /* Proceed to the next screen */
                aps.apsc.next(self.vpsModel.data);
            });
        },

        onPrev: function() {
            aps.apsc.prev(self.vpsModel.data);
        }

    });
});
